package repos

import (
	"context"
	"gralias/models"

	"github.com/jmoiron/sqlx"
)

type PlayersRepo interface {
	PlayerGetByName(ctx context.Context, username string) (*models.Player, error)
	PlayerAdd(ctx context.Context, player *models.Player) error
	PlayerUpdate(ctx context.Context, player *models.Player) error
	PlayerDelete(ctx context.Context, roomID, username string) error
	PlayerSetRoomID(ctx context.Context, username, roomID string) error
	PlayerExitRoom(ctx context.Context, username string) error
	PlayerListNames(ctx context.Context) ([]string, error)
	PlayerList(ctx context.Context, isBot bool) ([]models.Player, error)
}

func (p *RepoProvider) PlayerListNames(ctx context.Context) ([]string, error) {
	var names []string
	err := sqlx.SelectContext(ctx, p.DB, &names, "SELECT username FROM players;")
	if err != nil {
		return nil, err
	}
	return names, nil
}

func (p *RepoProvider) PlayerGetByName(ctx context.Context, username string) (*models.Player, error) {
	var player models.Player
	err := sqlx.GetContext(ctx, p.DB, &player, "SELECT id, room_id, username, team, role, is_bot FROM players WHERE username = ?", username)
	if err != nil {
		return nil, err
	}
	if player.RoomID != nil && *player.RoomID == "" {
		player.RoomID = nil
	}
	return &player, nil
}

func (p *RepoProvider) PlayerAdd(ctx context.Context, player *models.Player) error {
	db := getDB(ctx, p.DB)
	_, err := db.ExecContext(ctx, "INSERT INTO players (room_id, username, team, role, is_bot) VALUES (?, ?, ?, ?, ?)",
		player.RoomID, player.Username, player.Team, player.Role, player.IsBot)
	return err
}

func (p *RepoProvider) PlayerUpdate(ctx context.Context, player *models.Player) error {
	db := getDB(ctx, p.DB)
	_, err := db.ExecContext(ctx, "UPDATE players SET room_id = ?, username = ?, team = ?, role = ?, is_bot = ? WHERE id = ?",
		player.RoomID, player.Username, player.Team, player.Role, player.IsBot, player.ID)
	return err
}

func (p *RepoProvider) PlayerDelete(ctx context.Context, roomID, username string) error {
	db := getDB(ctx, p.DB)
	_, err := db.ExecContext(ctx, "DELETE FROM players WHERE room_id = ? AND username = ?", roomID, username)
	return err
}

func (p *RepoProvider) PlayerSetRoomID(ctx context.Context, username, roomID string) error {
	db := getDB(ctx, p.DB)
	_, err := db.ExecContext(ctx, "UPDATE players SET room_id = ? WHERE username = ?", roomID, username)
	return err
}

func (p *RepoProvider) PlayerExitRoom(ctx context.Context, username string) error {
	db := getDB(ctx, p.DB)
	_, err := db.ExecContext(ctx, "UPDATE players SET room_id='', team='', role='' WHERE username = ?", username)
	return err
}

func (p *RepoProvider) PlayerList(ctx context.Context, isBot bool) ([]models.Player, error) {
	var players []models.Player
	var query string
	if isBot {
		query = "SELECT id, room_id, username, team, role, is_bot FROM players WHERE is_bot = true;"
	} else {
		query = "SELECT id, room_id, username, team, role, is_bot FROM players WHERE is_bot = false;"
	}
	err := sqlx.SelectContext(ctx, p.DB, &players, query)
	if err != nil {
		return nil, err
	}
	return players, nil
}
